@@ -16,7 +16,7 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "../config.h"
+#include "config.h"
 
 #define _XOPEN_SOURCE 500
 #include <string.h>
@@ -24,9 +24,9 @@
 #include <dirent.h>
 #include <string.h>
 #include <librdf.h>
-#include <slv2/world.h>
-#include <slv2/slv2.h>
-#include <slv2/util.h>
+#include "slv2/world.h"
+#include "slv2/slv2.h"
+#include "slv2/util.h"
 #include "slv2_internal.h"
 
 
@@ -43,8 +43,15 @@ slv2_world_new()
 
 	librdf_world_open(world->world);
 	
-	world->storage = librdf_new_storage(world->world, "hashes", NULL,
-			"hash-type='memory'");
+	world->storage = librdf_new_storage(world->world, "trees", NULL, NULL);
+	if (!world->storage) {
+		fprintf(stderr, "Warning: Unable to create \"trees\" RDF storage.\n");
+		fprintf(stderr, "Performance can be improved by upgrading librdf.\n");
+		// Testing shows "hashes" to be faster than "memory" (list) here
+		world->storage = librdf_new_storage(world->world, "hashes", NULL,
+				"hash-type='memory'");
+	}
+
 	if (!world->storage)
 		goto fail;
 
@@ -56,26 +63,32 @@ slv2_world_new()
 	if (!world->parser)
 		goto fail;
 
-	static const char* lv2_plugin_uri = "http://lv2plug.in/ns/lv2core#Plugin";
-	world->lv2_plugin_class = slv2_plugin_class_new(world, NULL, lv2_plugin_uri, "Plugin");
-
 	world->plugin_classes = slv2_plugin_classes_new();
 	
 	world->plugins = slv2_plugins_new();
 	
 	world->lv2_specification_node = librdf_new_node_from_uri_string(world->world,
-			(unsigned char*)"http://lv2plug.in/ns/lv2core#Specification");
+			(const unsigned char*)"http://lv2plug.in/ns/lv2core#Specification");
 	
 	world->lv2_plugin_node = librdf_new_node_from_uri_string(world->world,
-			(unsigned char*)"http://lv2plug.in/ns/lv2core#Plugin");
+			(const unsigned char*)"http://lv2plug.in/ns/lv2core#Plugin");
 	
 	world->rdf_a_node = librdf_new_node_from_uri_string(world->world,
-			(unsigned char*)"http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+			(const unsigned char*)"http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+	
+	world->xsd_integer_node = librdf_new_node_from_uri_string(world->world,
+			(const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
+	
+	world->xsd_decimal_node = librdf_new_node_from_uri_string(world->world,
+			(const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
+
+	world->lv2_plugin_class = slv2_plugin_class_new(world, NULL,
+			librdf_node_get_uri(world->lv2_plugin_node), "Plugin");
 	
 	return world;
 
 fail:
-	free(world);
+	/* keep on rockin' in the */ free(world);
 	return NULL;
 }
 
@@ -91,8 +104,15 @@ slv2_world_new_using_rdf_world(librdf_world* rdf_world)
 	
 	world->local_world = false;
 
-	world->storage = librdf_new_storage(world->world, "hashes", NULL,
-			"hash-type='memory'");
+	world->storage = librdf_new_storage(world->world, "trees", NULL, NULL);
+	if (!world->storage) {
+		fprintf(stderr, "Warning: Unable to create \"trees\" RDF storage.\n");
+		fprintf(stderr, "Performance can be improved by upgrading librdf.\n");
+		// Testing shows "hashes" to be faster than "memory" (list) here
+		world->storage = librdf_new_storage(world->world, "hashes", NULL,
+				"hash-type='memory'");
+	}
+	
 	if (!world->storage)
 		goto fail;
 
@@ -104,26 +124,32 @@ slv2_world_new_using_rdf_world(librdf_world* rdf_world)
 	if (!world->parser)
 		goto fail;
 
-	static const char* lv2_plugin_uri = "http://lv2plug.in/ns/lv2core#Plugin";
-	world->lv2_plugin_class = slv2_plugin_class_new(world, NULL, lv2_plugin_uri, "Plugin");
-	
 	world->plugin_classes = slv2_plugin_classes_new();
 	
 	world->plugins = slv2_plugins_new();
 	
 	world->lv2_specification_node = librdf_new_node_from_uri_string(world->world,
-			(unsigned char*)"http://lv2plug.in/ns/lv2core#Specification");
+			(const unsigned char*)"http://lv2plug.in/ns/lv2core#Specification");
 	
-	world->lv2_plugin_node = librdf_new_node_from_uri_string(rdf_world,
-			(unsigned char*)"http://lv2plug.in/ns/lv2core#Plugin");
+	world->lv2_plugin_node = librdf_new_node_from_uri_string(world->world,
+			(const unsigned char*)"http://lv2plug.in/ns/lv2core#Plugin");
 	
 	world->rdf_a_node = librdf_new_node_from_uri_string(rdf_world,
-			(unsigned char*)"http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+			(const unsigned char*)"http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+	
+	world->xsd_integer_node = librdf_new_node_from_uri_string(world->world,
+			(const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
+	
+	world->xsd_decimal_node = librdf_new_node_from_uri_string(world->world,
+			(const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
+	
+	world->lv2_plugin_class = slv2_plugin_class_new(world, NULL,
+			librdf_node_get_uri(world->lv2_plugin_node), "Plugin");
 	
 	return world;
 
 fail:
-	free(world);
+	/* keep on rockin' in the */ free(world);
 	return NULL;
 }
 
@@ -131,9 +157,14 @@ fail:
 void
 slv2_world_free(SLV2World world)
 {
+	slv2_plugin_class_free(world->lv2_plugin_class);
+	world->lv2_plugin_class = NULL;
+
 	librdf_free_node(world->lv2_specification_node);
 	librdf_free_node(world->lv2_plugin_node);
 	librdf_free_node(world->rdf_a_node);
+	librdf_free_node(world->xsd_integer_node);
+	librdf_free_node(world->xsd_decimal_node);
 
 	for (int i=0; i < raptor_sequence_size(world->plugins); ++i)
 		slv2_plugin_free(raptor_sequence_get_at(world->plugins, i));
@@ -166,35 +197,22 @@ slv2_world_free(SLV2World world)
 void
 slv2_world_load_file(SLV2World world, librdf_uri* file_uri)
 {
-	librdf_storage* storage = librdf_new_storage(world->world, 
-			"memory", NULL, NULL);
-	librdf_model* model = librdf_new_model(world->world,
-			storage, NULL);
-	librdf_parser_parse_into_model(world->parser, file_uri, NULL, 
-			model);
-
-	librdf_stream* stream = librdf_model_as_stream(model);
-	librdf_model_add_statements(world->model, stream);
-	librdf_free_stream(stream);
-	
-	librdf_free_model(model);
-	librdf_free_storage(storage);
+	librdf_parser_parse_into_model(world->parser, file_uri, NULL, world->model);
 }
 
 
 
 void
-slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
+slv2_world_load_bundle(SLV2World world, SLV2Value bundle_uri)
 {
-	librdf_uri* bundle_uri = librdf_new_uri(world->world,
-			(const unsigned char*)bundle_uri_str);
-
 	librdf_uri* manifest_uri = librdf_new_uri_relative_to_base(
-			bundle_uri, (const unsigned char*)"manifest.ttl");
+			bundle_uri->val.uri_val, (const unsigned char*)"manifest.ttl");
 
 	/* Parse the manifest into a temporary model */
-	librdf_storage* manifest_storage = librdf_new_storage(world->world, 
-			"memory", NULL, NULL);
+	librdf_storage* manifest_storage = librdf_new_storage(world->world, "trees", NULL, NULL);
+	if (manifest_storage == NULL)
+		manifest_storage = librdf_new_storage(world->world, "memory", NULL, NULL);
+
 	librdf_model* manifest_model = librdf_new_model(world->world,
 			manifest_storage, NULL);
 	librdf_parser_parse_into_model(world->parser, manifest_uri, NULL, 
@@ -202,7 +220,8 @@ slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
 
 	/* Query statement: ?plugin a lv2:Plugin */
 	librdf_statement* q = librdf_new_statement_from_nodes(world->world,
-			NULL, world->rdf_a_node, world->lv2_plugin_node);
+			NULL, librdf_new_node_from_node(world->rdf_a_node),
+			      librdf_new_node_from_node(world->lv2_plugin_node));
 
 	librdf_stream* results = librdf_model_find_statements(manifest_model, q);
 
@@ -224,7 +243,7 @@ slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
 		subject = librdf_new_node_from_node(plugin_node);
 		predicate = librdf_new_node_from_uri_string(world->world, 
 				(unsigned char*)"http://drobilla.net/ns/slv2#bundleURI");
-		object = librdf_new_node_from_uri(world->world, bundle_uri);
+		object = librdf_new_node_from_uri(world->world, bundle_uri->val.uri_val);
 
 		librdf_model_add(world->model, subject, predicate, object);
 
@@ -232,11 +251,12 @@ slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
 	}
 	
 	librdf_free_stream(results);
-	free(q);
+	librdf_free_statement(q);
 	
 	/* Query statement: ?specification a lv2:Specification */
 	q = librdf_new_statement_from_nodes(world->world,
-			NULL, world->rdf_a_node, world->lv2_specification_node);
+			NULL, librdf_new_node_from_node(world->rdf_a_node),
+			      librdf_new_node_from_node(world->lv2_specification_node));
 
 	results = librdf_model_find_statements(manifest_model, q);
 
@@ -258,7 +278,7 @@ slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
 		subject = librdf_new_node_from_node(spec_node);
 		predicate = librdf_new_node_from_uri_string(world->world, 
 				(unsigned char*)"http://drobilla.net/ns/slv2#bundleURI");
-		object = librdf_new_node_from_uri(world->world, bundle_uri);
+		object = librdf_new_node_from_uri(world->world, bundle_uri->val.uri_val);
 
 		librdf_model_add(world->model, subject, predicate, object);
 
@@ -266,7 +286,7 @@ slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
 	}
 	
 	librdf_free_stream(results);
-	free(q);
+	librdf_free_statement(q);
 	
 	/* Join the temporary model to the main model */
 	librdf_stream* manifest_stream = librdf_model_as_stream(manifest_model);
@@ -276,7 +296,6 @@ slv2_world_load_bundle(SLV2World world, const char* bundle_uri_str)
 	librdf_free_model(manifest_model);
 	librdf_free_storage(manifest_storage);
 	librdf_free_uri(manifest_uri);
-	librdf_free_uri(bundle_uri);
 }
 
 
@@ -302,7 +321,9 @@ slv2_world_load_directory(SLV2World world, const char* dir)
 
 		if (bundle_dir != NULL) {
 			closedir(bundle_dir);
-			slv2_world_load_bundle(world, uri);
+			SLV2Value uri_val = slv2_value_new_uri(world, uri);
+			slv2_world_load_bundle(world, uri_val);
+			slv2_value_free(uri_val);
 		}
 		
 		free(uri);
@@ -335,6 +356,30 @@ slv2_world_load_path(SLV2World   world,
 }
 
 
+/** Comparator for sorting SLV2Plugins */
+int
+slv2_plugin_compare_by_uri(const void* a, const void* b)
+{
+    SLV2Plugin plugin_a = *(SLV2Plugin*)a;
+    SLV2Plugin plugin_b = *(SLV2Plugin*)b;
+
+    return strcmp(slv2_value_as_uri(plugin_a->plugin_uri),
+                  slv2_value_as_uri(plugin_b->plugin_uri));
+}
+
+
+/** Comparator for sorting SLV2PluginClasses */
+int
+slv2_plugin_class_compare_by_uri(const void* a, const void* b)
+{
+    SLV2PluginClass class_a = *(SLV2PluginClass*)a;
+    SLV2PluginClass class_b = *(SLV2PluginClass*)b;
+
+    return strcmp(slv2_value_as_uri(class_a->uri),
+                  slv2_value_as_uri(class_b->uri));
+}
+
+
 void
 slv2_world_load_specifications(SLV2World world)
 {
@@ -383,7 +428,7 @@ slv2_world_load_plugin_classes(SLV2World world)
 		"SELECT DISTINCT ?class ?parent ?label WHERE {\n"
 		//"	?plugin a ?class .\n"
 		"	?class a rdfs:Class; rdfs:subClassOf ?parent; rdfs:label ?label\n"
-		"} ORDER BY ?class\n";
+		"}\n"; // ORDER BY ?class\n";
 	
 	librdf_query* q = librdf_new_query(world->world, "sparql",
 		NULL, query_string, NULL);
@@ -401,10 +446,10 @@ slv2_world_load_plugin_classes(SLV2World world)
 		assert(class_uri);
 
 		SLV2PluginClass plugin_class = slv2_plugin_class_new(world,
-				parent_uri ? (const char*)librdf_uri_as_string(parent_uri) : NULL,
-				(const char*)librdf_uri_as_string(class_uri),
-				label);
+				parent_uri, class_uri, label);
 		raptor_sequence_push(world->plugin_classes, plugin_class);
+		// FIXME: Slow!  ORDER BY broken in certain versions of redland?
+		raptor_sequence_sort(world->plugin_classes, slv2_plugin_class_compare_by_uri);
 
 		librdf_free_node(class_node);
 		librdf_free_node(parent_node);
@@ -425,33 +470,26 @@ slv2_world_load_all(SLV2World world)
 {
 	char* lv2_path = getenv("LV2_PATH");
 
-	/* 1. Read LV2 ontology into model */
-	const char* ontology_path = "/usr/local/share/slv2/lv2.ttl";
-	FILE* ontology = fopen(ontology_path, "r");
-	if (ontology == NULL) {
-		ontology_path = "/usr/share/slv2/lv2.ttl";
-		ontology = fopen(ontology_path, "r");
-	}
-
-	if (ontology) {
-		fclose(ontology);
-		librdf_uri* ontology_uri = librdf_new_uri_from_filename(world->world,
-				ontology_path);
-		librdf_parser_parse_into_model(world->parser, ontology_uri, NULL, world->model);
-		librdf_free_uri(ontology_uri);
-	}
-
-	/* 2. Read all manifest files into model */
+	/* 1. Read all manifest files into model */
 
 	if (lv2_path) {
 		slv2_world_load_path(world, lv2_path);
 	} else {
 		const char* const home = getenv("HOME");
 		if (home) {
+#ifdef __APPLE__
+			const char* const suffix = "/Library/Audio/Plug-Ins/LV2:/Library/Audio/Plug-Ins/LV2"
+				":/usr/local/lib/lv2:/usr/lib/lv2";
+#else
 			const char* const suffix = "/.lv2:/usr/local/lib/lv2:/usr/lib/lv2";
+#endif
 			lv2_path = slv2_strjoin(home, suffix, NULL);
 		} else {
+#ifdef __APPLE__
+			lv2_path = strdup("/Library/Audio/Plug-Ins/LV2:/usr/local/lib/lv2:/usr/lib/lv2");
+#else
 			lv2_path = strdup("/usr/local/lib/lv2:/usr/lib/lv2");
+#endif
 		}
 
 		slv2_world_load_path(world, lv2_path);
@@ -460,7 +498,7 @@ slv2_world_load_all(SLV2World world)
 	}
 
 	
-	/* 3. Query out things to cache */
+	/* 2. Query out things to cache */
 
 	slv2_world_load_specifications(world);
 
@@ -471,10 +509,9 @@ slv2_world_load_all(SLV2World world)
     	"PREFIX : <http://lv2plug.in/ns/lv2core#>\n"
 		"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n"
 		"PREFIX slv2: <http://drobilla.net/ns/slv2#>\n"
-		"SELECT DISTINCT ?plugin ?data ?bundle ?binary\n"
-		"WHERE { ?plugin a :Plugin; slv2:bundleURI ?bundle; rdfs:seeAlso ?data\n"
-		"OPTIONAL { ?plugin :binary ?binary } }\n"
-		"ORDER BY ?plugin\n";
+		"SELECT DISTINCT ?plugin ?data ?bundle\n"
+		"WHERE { ?plugin a :Plugin; slv2:bundleURI ?bundle; rdfs:seeAlso ?data }\n";
+		//"ORDER BY ?plugin\n";
 	
 	librdf_query* q = librdf_new_query(world->world, "sparql",
 		NULL, query_string, NULL);
@@ -482,38 +519,39 @@ slv2_world_load_all(SLV2World world)
 	librdf_query_results* results = librdf_query_execute(q, world->model);
 
 	while (!librdf_query_results_finished(results)) {
-	
+
 		librdf_node* plugin_node = librdf_query_results_get_binding_value(results, 0);
 		librdf_uri*  plugin_uri  = librdf_node_get_uri(plugin_node);
 		librdf_node* data_node   = librdf_query_results_get_binding_value(results, 1);
 		librdf_uri*  data_uri    = librdf_node_get_uri(data_node);
 		librdf_node* bundle_node = librdf_query_results_get_binding_value(results, 2);
 		librdf_uri*  bundle_uri  = librdf_node_get_uri(bundle_node);
-		librdf_node* binary_node = librdf_query_results_get_binding_value(results, 3);
-		librdf_uri*  binary_uri  = librdf_node_get_uri(binary_node);
-		
+
 		assert(plugin_uri);
 		assert(data_uri);
 
-		SLV2Plugin plugin = slv2_plugins_get_by_uri(world->plugins,
-				(const char*)librdf_uri_as_string(plugin_uri));
-		
+		SLV2Value   uri        = slv2_value_new_librdf_uri(world, plugin_uri);
+		SLV2Plugin  plugin     = slv2_plugins_get_by_uri(world->plugins, uri);
+
 		// Create a new SLV2Plugin
 		if (!plugin) {
-			plugin = slv2_plugin_new(world, plugin_uri, bundle_uri, binary_uri);
+			plugin = slv2_plugin_new(world, uri, bundle_uri);
 			raptor_sequence_push(world->plugins, plugin);
+			// FIXME: Slow!  ORDER BY broken in certain versions of redland?
+			raptor_sequence_sort(world->plugins, slv2_plugin_compare_by_uri);
+		} else {
+			slv2_value_free(uri);
 		}
-		
+
 		plugin->world = world;
 
 		// FIXME: check for duplicates
 		raptor_sequence_push(plugin->data_uris,
-				slv2_value_new(SLV2_VALUE_URI, (const char*)librdf_uri_as_string(data_uri)));
-		
+				slv2_value_new_librdf_uri(plugin->world, data_uri));
+
 		librdf_free_node(plugin_node);
 		librdf_free_node(data_node);
 		librdf_free_node(bundle_node);
-		librdf_free_node(binary_node);
 
 		librdf_query_results_next(results);
 	}
