@@ -27,6 +27,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "defines.h"
 #include "sndfile.h"
 
+#include <QFile>
+
 class QString;
 
 class SFAudioWriter : public AbstractAudioWriter
@@ -48,6 +50,10 @@ protected:
 	int		m_fileType;	
 	SNDFILE*	m_sf;
 	SF_INFO 	m_sfinfo;
+	
+private:
+	QFile m_file;
+
 };
 
 #endif
