@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005-2007 Remon Sijrier 
+Copyright (C) 2005-2008 Remon Sijrier 
 
 This file is part of Traverso
 
@@ -23,13 +23,14 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "ClipSelection.h"
 #include "AudioClipManager.h"
 #include <AudioClip.h>
-#include <Song.h>
+#include "ContextPointer.h"
+#include <Sheet.h>
 #include <Utils.h>
 
 #include "Debugger.h"
 
 ClipSelection::ClipSelection(AudioClip* clip, QVariantList args)
-	: Command(clip, "")
+	: Command("")
 {
 	QString slot = args.at(0).toString();
 	
@@ -43,11 +44,11 @@ ClipSelection::ClipSelection(AudioClip* clip, QVariantList args)
 	
 	m_clips.append( clip );
 	m_slot = qstrdup(QS_C(slot));
-	m_acmanager = clip->get_song()->get_audioclip_manager();
+	m_acmanager = clip->get_sheet()->get_audioclip_manager();
 }
 
 ClipSelection::ClipSelection( QList< AudioClip * > clips, AudioClipManager * manager, const char * slot, const QString& des )
-		: Command(manager, des)
+	: Command("")
 {
 	m_clips = clips;
 	m_slot = slot;
@@ -57,10 +58,18 @@ ClipSelection::ClipSelection( QList< AudioClip * > clips, AudioClipManager * man
 ClipSelection::~ClipSelection()
 {}
 
+int ClipSelection::begin_hold()
+{
+	return -1;
+}
+
+int ClipSelection::finish_hold()
+{
+	return -1;
+}
+
 int ClipSelection::prepare_actions()
 {
-	m_acmanager->get_selected_clips_state( selectedClips );
-	
 	return 1;
 }
 
@@ -77,10 +86,15 @@ int ClipSelection::do_action()
 
 int ClipSelection::undo_action()
 {
-	m_acmanager->set_selected_clips_state( selectedClips );
-	
 	return 1;
 }
 
-// eof
+void ClipSelection::cancel_action()
+{
+	undo_action();
+}
 
+int ClipSelection::jog()
+{
+	return -1;
+}
