@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: Gain.cpp,v 1.23 2007/11/27 20:20:42 r_sijrier Exp $
+$Id: Gain.cpp,v 1.27 2008/02/13 10:25:25 r_sijrier Exp $
 */
 
 #include "Gain.h"
@@ -27,15 +27,22 @@ $Id: Gain.cpp,v 1.23 2007/11/27 20:20:42 r_sijrier Exp $
 #include "Mixer.h"
 #include <ViewPort.h>
 #include <Track.h>
-#include "Song.h"
-#include "SongView.h"
+#include "Sheet.h"
+#include "SheetView.h"
 
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
 #include "Debugger.h"
 
+/**
+ *	\class Gain
+	\brief Change (jog) the Gain of a Sheet, Track or AudioClip, or set to a pre-defined value
+	
+	\sa TraversoCommands
+ */
 
-Gain::Gain(ContextItem* context, SongView* sv, QVariantList args)
+
+Gain::Gain(ContextItem* context, SheetView* sv, QVariantList args)
 	: Command(context, "")
 	, m_sv(sv)
 {
@@ -67,8 +74,8 @@ Gain::Gain(ContextItem* context, SongView* sv, QVariantList args)
 	if (track && origGain == 0.5) {
 		newGain = 1.0;
 	} else {
-		Song* song = qobject_cast<Song*>(context);
-		if (song && origGain == 0.5) {
+		Sheet* sheet = qobject_cast<Sheet*>(context);
+		if (sheet && origGain == 0.5) {
 			newGain = 1.0;
 		}
 	}
@@ -132,6 +139,45 @@ void Gain::cancel_action()
 	undo_action();
 }
 
+void Gain::set_collected_number(const QString & collected)
+{
+	if (collected.size() == 0) {
+		cpointer().get_viewport()->set_holdcursor_text(" dB");
+		return;
+	}
+	
+	bool ok;
+	float dbFactor = collected.toDouble(&ok);
+	if (!ok) {
+		if (collected.contains(".") || collected.contains("-")) {
+			QString s = collected;
+			s.append(" dB");
+			cpointer().get_viewport()->set_holdcursor_text(s);
+		}
+		return;
+	}
+	
+	int rightfromdot = 0;
+	if (collected.contains(".")) {
+		rightfromdot = collected.size() - collected.lastIndexOf(".") - 1;
+	}
+	
+	newGain = dB_to_scale_factor(dbFactor);
+	QMetaObject::invokeMethod(gainObject, "set_gain", Q_ARG(float, newGain));
+	
+	// now we get the new gain value from gainObject, since we don't know if 
+	// gainobject accepted the change or not!
+	get_gain_from_object(newGain);
+	
+	// Update the vieport's hold cursor with the _actuall_ gain value!
+	if(rightfromdot) {
+		cpointer().get_viewport()->set_holdcursor_text(QByteArray::number(dbFactor, 'f', rightfromdot).append(" dB"));
+	} else {
+		cpointer().get_viewport()->set_holdcursor_text(QByteArray::number(dbFactor).append(" dB"));
+	}
+
+}
+
 
 void Gain::set_cursor_shape(int useX, int useY)
 {
@@ -249,6 +295,3 @@ int Gain::get_gain_from_object(float& gain)
 	return 1;
 }
 
-
-// eof
-
