@@ -19,5 +19,10 @@
 #include <qstyle.h>
 #include <qtimer.h>
 #include <qwidget.h>
-
+#include <QDomDocument>
+#include <QDomNode>
+#include <QHash>
+#include <QWheelEvent>
+#include <QUndoGroup>
+#include <qgraphicsview.h>
 #include <stdlib.h>
