@@ -1,23 +1,23 @@
 /*
-    Copyright (C) 2005-2006 Remon Sijrier 
- 
+    Copyright (C) 2005-2006 Remon Sijrier
+
     This file is part of Traverso
- 
+
     Traverso is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
- 
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
- 
+
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
- 
-    $Id: AudioChannel.cpp,v 1.5 2006/07/05 11:11:15 r_sijrier Exp $
+
+    $Id: AudioChannel.cpp,v 1.7 2006/09/07 09:36:52 r_sijrier Exp $
 */
 
 #include "AudioChannel.h"
@@ -32,7 +32,16 @@
 // in case we run with memory leak detection enabled!
 #include "Debugger.h"
 
-AudioChannel::AudioChannel( QString busName, QString audioType, int flags, uint channelNumber )
+/**
+ * \class AudioChannel
+ * \brief AudioChannel wraps one 'real' audiochannel into an easy to use class
+ *
+ * An AudioChannel has a audio_sample_t* buffer, a name and some functions for setting the buffer size, 
+ * and monitoring the highest peak value, which is handy to use by for example a VU meter. 
+ */
+
+
+AudioChannel::AudioChannel( const QString& busName, const QString& audioType, int flags, uint channelNumber )
 {
         m_name = busName;
         m_audioType = audioType;
@@ -97,6 +106,11 @@ void AudioChannel::set_buffer_size( nframes_t size )
 }
 
 
+/**
+ * 
+ * @return The highest peak value since the previous call to this function,
+ *		 call this at least 10 times each second to keep data consistent 
+ */
 audio_sample_t AudioChannel::get_peak_value( )
 {
         float peak = 0;
@@ -117,4 +131,3 @@ void AudioChannel::set_monitor_peaks( bool monitor )
 	monitoring = monitor;
 }
 //eof
-
