@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: AudioDevice.cpp,v 1.49 2007/12/07 11:22:32 r_sijrier Exp $
+$Id: AudioDevice.cpp,v 1.56 2009/03/04 20:54:00 r_sijrier Exp $
 */
 
 #include "AudioDevice.h"
@@ -36,12 +36,20 @@ RELAYTOOL_JACK
 #include "PADriver.h"
 #endif
 
+#if defined (PULSEAUDIO_SUPPORT)
+#include "PulseAudioDriver.h"
+#endif
+
+#if defined (COREAUDIO_SUPPORT)
+#include "CoreAudioDriver.h"
+#endif
+
+
 #include "Driver.h"
 #include "Client.h"
 #include "AudioChannel.h"
 #include "AudioBus.h"
 #include "Tsar.h"
-
 //#include <sys/mman.h>
 
 // Always put me below _all_ includes, this is needed
@@ -162,6 +170,15 @@ AudioDevice::AudioDevice()
 	availableDrivers << "PortAudio";
 #endif
 
+#if defined (PULSEAUDIO_SUPPORT)
+	availableDrivers << "PulseAudio";
+#endif
+
+#if defined (COREAUDIO_SUPPORT)
+	availableDrivers << "CoreAudio";
+#endif
+
+	
 	availableDrivers << "Null Driver";
 	
 	// tsar is a singleton, so initialization is done on first tsar() call
@@ -320,13 +337,15 @@ void AudioDevice::set_parameters( int rate,
 				const QString& driverType,
 				bool capture,
 				bool playback,
-				const QString& cardDevice)
+				const QString& cardDevice,
+				const QString& ditherShape)
 {
 	PENTER;
 
 	m_rate = rate;
 	m_bufferSize = bufferSize;
 	m_xrunCount = 0;
+	m_ditherShape = ditherShape;
 
 	shutdown();
 
@@ -388,7 +407,7 @@ void AudioDevice::set_parameters( int rate,
 	}
 #endif
 		
-	if (driverType == "PortAudio") {
+	if (driverType == "PortAudio"|| (driverType == "PulseAudio") || (driverType == "CoreAudio")) {
 		if (driver->start() == -1) {
 			// PortAudio driver failed to start, fallback to Null Driver:
 			set_parameters(rate, bufferSize, "Null Driver");
@@ -421,7 +440,7 @@ int AudioDevice::create_driver(QString driverType, bool capture, bool playback,
 #if defined (ALSA_SUPPORT)
 	if (driverType == "ALSA") {
 		driver =  new AlsaDriver(this, m_rate, m_bufferSize);
-		if (driver->setup(capture,playback, cardDevice) < 0) {
+		if (((AlsaDriver*)driver)->setup(capture,playback, cardDevice, m_ditherShape) < 0) {
 			message(tr("Audiodevice: Failed to create the ALSA Driver"), WARNING);
 			delete driver;
 			driver = 0;
@@ -446,6 +465,36 @@ int AudioDevice::create_driver(QString driverType, bool capture, bool playback,
 	}
 #endif
 	
+#if defined (PULSEAUDIO_SUPPORT)
+	if (driverType == "PulseAudio") {
+		driver = new PulseAudioDriver(this, m_rate, m_bufferSize);
+		if (driver->setup(capture, playback, cardDevice) < 0) {
+			message(tr("Audiodevice: Failed to create the PulseAudio Driver"), WARNING);
+			delete driver;
+			driver = 0;
+			return -1;
+		}
+		m_driverType = driverType;
+		return 1;
+	}
+#endif
+
+
+#if defined (COREAUDIO_SUPPORT)
+	if (driverType == "CoreAudio") {
+		driver = new CoreAudioDriver(this, m_rate, m_bufferSize);
+		if (driver->setup(capture, playback, cardDevice) < 0) {
+			message(tr("Audiodevice: Failed to create the CoreAudio Driver"), WARNING);
+			delete driver;
+			driver = 0;
+			return -1;
+		}
+		m_driverType = driverType;
+		return 1;
+	}
+#endif
+
+	
 	if (driverType == "Null Driver") {
 		printf("Creating Null Driver...\n");
 		driver = new Driver(this, m_rate, m_bufferSize);
@@ -552,11 +601,19 @@ void AudioDevice::setup_buses( )
 	int number = 1;
 	QByteArray name;
 
+	AudioChannel* channel;
+	
 	for (int i=1; i <= captureChannels.size();) {
 		name = "Capture " + QByteArray::number(number++);
 		AudioBus* bus = new AudioBus(name);
-		bus->add_channel(captureChannels.value("capture_"+QByteArray::number(i++)));
-		bus->add_channel(captureChannels.value("capture_"+QByteArray::number(i++)));
+		channel = captureChannels.value("capture_"+QByteArray::number(i++), 0);
+		if (channel) {
+			bus->add_channel(channel);
+		}
+		channel = captureChannels.value("capture_"+QByteArray::number(i++), 0);
+		if (channel) {
+			bus->add_channel(channel);
+		}
 		captureBuses.insert(name, bus);
 	}
 // 	PWARN("Capture buses count is: %d", captureBuses.size());
@@ -566,8 +623,14 @@ void AudioDevice::setup_buses( )
 	for (int i=1; i <= playbackChannels.size();) {
 		name = "Playback " + QByteArray::number(number++);
 		AudioBus* bus = new AudioBus(name);
-		bus->add_channel(playbackChannels.value("playback_"+QByteArray::number(i++)));
-		bus->add_channel(playbackChannels.value("playback_"+QByteArray::number(i++)));
+		channel = playbackChannels.value("playback_"+QByteArray::number(i++), 0);
+		if (channel) {
+			bus->add_channel(channel);
+		}
+		channel = playbackChannels.value("playback_"+QByteArray::number(i++), 0);
+		if (channel) {
+			bus->add_channel(channel);
+		}
 		playbackBuses.insert(name, bus);
 	}
 // 	PWARN("Playback buses count is: %d", playbackBuses.size());
@@ -824,7 +887,7 @@ int AudioDevice::transport_seek_to(Client* client, TimeRef location)
 	if (jackdriver) {
 		PMESG("using jack_transport_locate");
 		nframes_t frames = location.to_frame(get_sample_rate());
-		return  jack_transport_locate(jackdriver->get_client(), frames);
+		return jack_transport_locate(jackdriver->get_client(), frames);
 	}
 #endif
 	
