@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005-2006 Remon Sijrier 
+Copyright (C) 2005-2007 Remon Sijrier
 
 This file is part of Traverso
 
@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: AudioDeviceThread.cpp,v 1.6 2006/07/28 12:46:28 r_sijrier Exp $
+$Id: AudioDeviceThread.cpp,v 1.21 2007/10/20 17:38:19 r_sijrier Exp $
 */
 
 #include "AudioDeviceThread.h"
@@ -25,8 +25,12 @@ $Id: AudioDeviceThread.cpp,v 1.6 2006/07/28 12:46:28 r_sijrier Exp $
 #include "AudioDevice.h"
 #include "Driver.h"
 
+#if defined (Q_WS_X11)
+#include <dlfcn.h>
 #include <sys/resource.h>
 #include <sched.h>
+#endif
+
 #include <sys/time.h>
 #include <unistd.h>
 #include <signal.h>
@@ -48,9 +52,12 @@ public:
 protected:
 	void run()
 	{
+#if defined (Q_WS_X11) || defined (Q_WS_MAC)
 		struct sched_param param;
 		param.sched_priority = 90;
 		if (pthread_setschedparam (pthread_self(), SCHED_FIFO, &param) != 0) {}
+#endif
+
 
 		while(true) {
 			sleep(5);
@@ -58,8 +65,10 @@ protected:
 
 			if (guardedThread->watchdogCheck == 0) {
 				qCritical("WatchDog timed out!");
-				guardedThread->terminate();
+//				guardedThread->terminate();
+#if defined (Q_WS_X11) || defined (Q_WS_MAC)
 				kill (-getpgrp(), SIGABRT);
+#endif
 			}
 
 			guardedThread->watchdogCheck = 0;
@@ -70,14 +79,13 @@ protected:
 AudioDeviceThread::AudioDeviceThread(AudioDevice* device)
 {
 	m_device = device;
-	transfer = false;
-	realTime = true;
+	m_realTime = true;
 	setTerminationEnabled(true);
-	
-#ifndef MAC_OS_BUILD
-	setStackSize(100000);
+
+#ifndef Q_WS_MAC
+	setStackSize(1000000);
 #endif
-	
+
 	watchdogCheck = 1;
 }
 
@@ -85,72 +93,84 @@ AudioDeviceThread::AudioDeviceThread(AudioDevice* device)
 void AudioDeviceThread::run()
 {
 	run_on_cpu( 0 );
-	become_realtime(true);
-	int err =0;
 
+	
 	WatchDogThread watchdog(this);
 	watchdog.start();
 
+	become_realtime(m_realTime);
+	
+	if (m_device->driver->start() < 0) {
+		watchdog.terminate();
+		watchdog.wait();
+		return;
+	}
+
 	while (m_device->run_audio_thread()) {
-		if ((err = m_device->get_driver()->run_cycle()) < 0) {
+		if (m_device->get_driver()->run_cycle() < 0) {
 			PERROR("Driver cycle error, exiting!");
 			break;
 		}
 		watchdogCheck = 1;
 	}
+	
 	watchdog.terminate();
 	watchdog.wait();
 }
 
 
-int AudioDeviceThread::transfer_start( )
-{
-	transfer = true;
-	return 1;
-}
-
-int AudioDeviceThread::transfer_stop( )
-{
-	transfer = false;
-	return 1;
-}
-
-void AudioDeviceThread::start_transfering( )
-{}
-
 int AudioDeviceThread::become_realtime( bool realtime )
 {
+	m_realTime = realtime;
+#if defined (Q_WS_X11) || defined (Q_WS_MAC)
+
 	/* RTC stuff */
 	if (realtime) {
 		struct sched_param param;
 		param.sched_priority = 70;
 		if (pthread_setschedparam (pthread_self(), SCHED_FIFO, &param) != 0) {
-			qWarning("Unable to set Audiodevice Thread to realtime priority!!!\n"
-				 "This most likely results in unreliable playback/capture and\n"
-				 "lots of buffer underruns (== sound drops).\n"
-				 "In the worst case the program can even abort!\n"
-				 "Please make sure you run this program with realtime privileges!!!\n");
+			m_device->message(tr("Unable to set Audiodevice Thread to realtime priority!!!"
+				"This most likely results in unreliable playback/capture and "
+				"lots of buffer underruns (== sound drops)."
+				"In the worst case the program can even malfunction!"
+				"Please make sure you run this program with realtime privileges!!!"), AudioDevice::CRITICAL);
 			return -1;
 		} else {
-			qDebug("Running realtime");
+			printf("AudioThread: Running with realtime priority\n");
 			return 1;
 		}
 	}
+
+#endif
+
 	return -1;
 }
 
+
+#if defined (Q_WS_X11)
+typedef int* (*setaffinity_func_type)(pid_t,unsigned int,cpu_set_t *);
+#endif
+
 void AudioDeviceThread::run_on_cpu( int cpu )
 {
-#if defined (MAC_OS_BUILD)
-	return;
-#else
-	cpu_set_t mask;
-	CPU_ZERO(&mask);
-	CPU_SET(cpu, &mask);
-	if (sched_setaffinity(0, sizeof(mask), &mask)) {
-		PWARN("Unable to set CPU affinity");
-	} else {
-		PMESG("Running AudioDeviceThread on CPU %d", cpu);
+#if defined (Q_WS_X11)
+	void *setaffinity_handle = dlopen(NULL, RTLD_LAZY);// NULL might not be portable to platforms other than linux - tajmorton@gmail.com
+	
+	setaffinity_func_type setaffinity_func;
+	setaffinity_func = (setaffinity_func_type) dlsym(setaffinity_handle, "sched_setaffinity");
+	
+	if (setaffinity_func != NULL) {
+		cpu_set_t mask;
+		CPU_ZERO(&mask);
+		CPU_SET(cpu, &mask);
+		if (setaffinity_func(0, sizeof(mask), &mask)) {
+			PWARN("Unable to set CPU affinity\n");
+		} else {
+			PMESG("Running AudioDeviceThread on CPU %d\n", cpu);
+		}
+	}
+	else {
+		PWARN("Unable to set CPU affinity (glibc is too old)\n");
 	}
 #endif
 }
