@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005-2006 Remon Sijrier 
+Copyright (C) 2005-2007 Remon Sijrier 
 
 This file is part of Traverso
 
@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: Client.h,v 1.2 2006/06/26 23:58:13 r_sijrier Exp $
+$Id: Client.h,v 1.7 2007/11/19 11:18:54 r_sijrier Exp $
 */
 
 #ifndef CLIENT_H
@@ -25,21 +25,25 @@ $Id: Client.h,v 1.2 2006/06/26 23:58:13 r_sijrier Exp $
 
 #include <QString>
 #include <QObject>
+#include "APILinkedList.h"
 
 #include "defines.h"
 
-class Client : public QObject
+class Client : public QObject, public APILinkedListNode
 {
 	Q_OBJECT
 
 public:
-	Client(QString name);
+	Client(const QString& name);
 	~Client();
 
 	void set_process_callback(ProcessCallback call);
+	void set_transport_control_callback(TransportControlCallback call);
+	bool is_smaller_then(APILinkedListNode* ) {return false;}
 
 	
 	ProcessCallback process;
+	TransportControlCallback transport_control;
 	
 	QString		m_name;
 
