@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006 Remon Sijrier 
+    Copyright (C) 2005-2007 Remon Sijrier 
  
     This file is part of Traverso
  
@@ -17,20 +17,20 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  
-    $Id: JackDriver.cpp,v 1.3 2006/07/04 20:38:48 r_sijrier Exp $
+    $Id: JackDriver.cpp,v 1.24 2007/12/07 13:21:49 r_sijrier Exp $
 */
 
 #include "JackDriver.h"
 
-#include "AudioDevice.h"
-#include "AudioChannel.h"
-
 #include <jack/jack.h>
 
 #if defined (ALSA_SUPPORT)
 #include "AlsaDriver.h"
 #endif
 
+#include "AudioDevice.h"
+#include "AudioChannel.h"
+
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
 #include "Debugger.h"
@@ -41,28 +41,27 @@ JackDriver::JackDriver( AudioDevice * dev , int rate, nframes_t bufferSize)
         read = MakeDelegate(this, &JackDriver::_read);
         write = MakeDelegate(this, &JackDriver::_write);
         run_cycle = RunCycleCallback(this, &JackDriver::_run_cycle);
+	m_running = false;
 }
 
 JackDriver::~JackDriver( )
 {
-        if (client)
+	PENTER;
+	if (m_running == 0) {
                 jack_client_close (client);
-}
-
-int JackDriver::_run_cycle( )
-{
-        return 1;
+	}
 }
 
 int JackDriver::_read( nframes_t nframes )
 {
         int portNumber = 0;
-        foreach(AudioChannel* bus, captureChannels) {
-                if (!bus->has_data()) {
+	for (int i=0; i<captureChannels.size(); ++i) {
+		AudioChannel* chan = captureChannels.at(i);
+                if (!chan->has_data()) {
                         portNumber++;
                         continue;
                 }
-                memcpy (bus->get_data(), jack_port_get_buffer (inputPorts[portNumber], nframes), sizeof (jack_default_audio_sample_t) * nframes);
+                memcpy (chan->get_data(), jack_port_get_buffer (inputPorts[portNumber], nframes), sizeof (jack_default_audio_sample_t) * nframes);
                 portNumber++;
         }
         return 1;
@@ -71,7 +70,9 @@ int JackDriver::_read( nframes_t nframes )
 int JackDriver::_write( nframes_t nframes )
 {
         int portNumber = 0;
-        foreach(AudioChannel* chan, playbackChannels) {
+	for (int i=0; i<playbackChannels.size(); ++i) {
+		AudioChannel* chan = playbackChannels.at(i);
+		
 /*		if (!chan->has_data()) {
 			portNumber++;
 			continue;
@@ -83,13 +84,13 @@ int JackDriver::_write( nframes_t nframes )
         return 1;
 }
 
-int JackDriver::_null_cycle( nframes_t )
+int JackDriver::setup(bool capture, bool playback, const QString& )
 {
-        return 1;
-}
+	PENTER;
+	
+	Q_UNUSED(capture);
+	Q_UNUSED(playback);
 
-int JackDriver::setup( )
-{
         const char **inputports;
         const char **outputports;
         const char *client_name = "Traverso";
@@ -97,7 +98,7 @@ int JackDriver::setup( )
         int inputPortCount = 0;
         int outputPortCount = 0;
         client = 0;
-        AudioChannel* bus;
+        AudioChannel* channel;
         char buf[32];
         int port_flags;
         capture_frame_latency = playback_frame_latency =0;
@@ -106,7 +107,7 @@ int JackDriver::setup( )
         printf("Connecting to the Jack server...\n");
 
         if ( (client = jack_client_new (client_name)) == NULL) {
-                PWARN("Could not connect to Jack server, Jack not running?");
+		device->message(tr("Jack Driver: Couldn't connect to the jack server, is jack running?"), AudioDevice::WARNING);
                 return -1;
         }
 
@@ -176,9 +177,9 @@ int JackDriver::setup( )
 
                 snprintf (buf, sizeof(buf) - 1, "capture_%d", chn+1);
 
-                bus = device->register_capture_channel(buf, JACK_DEFAULT_AUDIO_TYPE, port_flags, frames_per_cycle, chn);
-                bus->set_latency( frames_per_cycle + capture_frame_latency );
-                captureChannels.append(bus);
+		channel = device->register_capture_channel(buf, JACK_DEFAULT_AUDIO_TYPE, port_flags, frames_per_cycle, chn);
+		channel->set_latency( frames_per_cycle + capture_frame_latency );
+		captureChannels.append(channel);
         }
 
 
@@ -187,9 +188,9 @@ int JackDriver::setup( )
 
                 snprintf (buf, sizeof(buf) - 1, "playback_%d", chn+1);
 
-                bus = device->register_playback_channel(buf, JACK_DEFAULT_AUDIO_TYPE, port_flags, frames_per_cycle, chn);
-                bus->set_latency( frames_per_cycle + capture_frame_latency );
-                playbackChannels.append(bus);
+		channel = device->register_playback_channel(buf, JACK_DEFAULT_AUDIO_TYPE, port_flags, frames_per_cycle, chn);
+		channel->set_latency( frames_per_cycle + capture_frame_latency );
+		playbackChannels.append(channel);
         }
 
 
@@ -199,34 +200,68 @@ int JackDriver::setup( )
         jack_set_process_callback (client, _process_callback, this);
         jack_set_xrun_callback (client, _xrun_callback, this);
         jack_set_buffer_size_callback (client, _bufsize_callback, this);
+	jack_on_shutdown(client, _on_jack_shutdown_callback, this);
 
+	update_config();
 
-        printf("Connected to the Jack server succesfully!\n");
-        return 1;
+	device->message(tr("Jack Driver: Connected successfully to the jack server!"), AudioDevice::INFO);
+        
+	return 1;
 }
 
 int JackDriver::attach( )
 {
+	PENTER;
         return 1;
 }
 
 int JackDriver::start( )
 {
-        jack_activate (client);
-        return 1;
+	PENTER;
+        if (jack_activate (client)) {
+		//if jack_active() != 0, something went wrong!
+		return -1;
+	}
+	
+	m_running = 1;
+	return 1;
 }
 
 int JackDriver::stop( )
 {
-        return 1;
+	PENTER;
+	m_running = 0;
+	return 1;
 }
 
 int JackDriver::process_callback (nframes_t nframes)
 {
-        device->run_cycle( nframes, 0.0);
-        return 1;
+	jack_position_t pos;
+	jack_transport_state_t state = jack_transport_query (client, &pos);
+	
+	transport_state_t tranportstate;
+	tranportstate.tranport = state;
+	tranportstate.location = TimeRef(pos.frame, audiodevice().get_sample_rate());
+	tranportstate.realtime = true;
+	
+	device->transport_control(tranportstate);
+	
+	device->run_cycle( nframes, 0.0);
+        return 0;
 }
 
+int JackDriver::jack_sync_callback (jack_transport_state_t state, jack_position_t* pos)
+{
+	transport_state_t tranportstate;
+	tranportstate.tranport = state;
+	tranportstate.location = TimeRef(pos->frame, audiodevice().get_sample_rate());
+	tranportstate.isSlave = true;
+	tranportstate.realtime = true;
+	
+	return device->transport_control(tranportstate);
+}
+
+
 // Is there a way to get the device name from Jack? Can't find it :-(
 // Since Jack uses ALSA, we ask it from ALSA directly :-)
 QString JackDriver::get_device_name( )
@@ -248,13 +283,20 @@ QString JackDriver::get_device_longname( )
 int JackDriver::_xrun_callback( void * arg )
 {
         JackDriver* driver  = static_cast<JackDriver *> (arg);
-        emit driver->device->xrun();
+	if (driver->m_running) {
+        	driver->device->xrun();
+	}
         return 0;
 }
 
 int JackDriver::_process_callback (nframes_t nframes, void *arg)
 {
-        return static_cast<JackDriver *> (arg)->process_callback (nframes);
+	JackDriver* driver  = static_cast<JackDriver *> (arg);
+	if (!driver->m_running) {
+		return 0;
+	}
+	
+	return driver->process_callback (nframes);
 }
 
 int JackDriver::_bufsize_callback( nframes_t nframes, void * arg )
@@ -271,5 +313,27 @@ float JackDriver::get_cpu_load( )
         return jack_cpu_load(client);
 }
 
+void JackDriver::_on_jack_shutdown_callback( void * arg )
+{
+	JackDriver* driver  = static_cast<JackDriver *> (arg);
+	driver->m_running = -1;
+}
+
+int JackDriver::_jack_sync_callback (jack_transport_state_t state, jack_position_t* pos, void* arg)
+{
+	return static_cast<JackDriver*> (arg)->jack_sync_callback (state, pos);
+}
+
+void JackDriver::update_config()
+{
+	m_isSlave = device->get_driver_property("jackslave", false).toBool();
+		
+	if (m_isSlave) {
+		jack_set_sync_callback (client, _jack_sync_callback, this);
+	} else {
+		jack_set_sync_callback(client, NULL, this);
+	}
+}
 
 //eof
+
