@@ -1,32 +1,36 @@
 /*
-	Copyright (C) 2005-2006 Remon Sijrier
+Copyright (C) 2005-2009 Remon Sijrier 
 
-	This file is part of Traverso
+This file is part of Traverso
 
-	Traverso is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
-	(at your option) any later version.
+Traverso is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
 */
 
 #include <signal.h>
+#include <stdlib.h>
 
 #include <QLocale>
 #include <QTranslator>
 #include <QtPlugin>
+
+#include "Config.h"
 #include "Traverso.h"
 #include "Main.h"
 #include "../config.h"
+#include <cstdlib>
 
 // Always put me below _all_ includes, this is needed
 // in case we run with memory leak detection enabled!
@@ -113,10 +117,17 @@ int main( int argc, char **argv )
 	traverso = new Traverso(argc, argv);
 	
 	QTranslator traversoTranslator;
-	QString language = QLocale::system().name();
-	traversoTranslator.load(":/traverso_" + language );
+	QString systemLanguage = QLocale::system().name();
+	QString userLanguage = config().get_property("Interface", "LanguageFile", "").toString();
+	if (userLanguage.isEmpty() || userLanguage.isNull()) {
+		traversoTranslator.load(":/translations/traverso_" + systemLanguage );
+	} else {
+		traversoTranslator.load(userLanguage);
+	}
 	traverso->installTranslator(&traversoTranslator);
+	
 	traverso->exec();
+	
 	delete traverso;
 
 	MEM_OFF();
